
#include <string.h>
#include <stdio.h>

/* FreeRTOS */
#include "FreeRTOS.h"
#include "os_queue.h"
#include "os_task.h"
#include "os_semphr.h"

/* TMS570 */
#include "sci.h"

/* main app */
#include "platform.h"
#include "tms570_configuration.h"
#include "globals.h"


#include "ti_fee.h"

#define portRESET_PRIVILEGE( xRunningPrivileged ) if( xRunningPrivileged == 0 ) portSWITCH_TO_USER_MODE()

#pragma SWI_ALIAS(prvRaisePrivilege, 1);
extern portBASE_TYPE prvRaisePrivilege( void );

char respBuffer[200];

void DelayMs( uint32_t ms )
{
	portTickType delay = ms / portTICK_RATE_MS;
	vTaskDelay( delay );
}


void fapi_wait(void)
{
	boolean done = false;
	portBASE_TYPE xRunningPrivileged;

	do {
		serial_console_print(".");
		DelayMs(500);
		portDISABLE_INTERRUPTS();
		xRunningPrivileged = prvRaisePrivilege();
		if (FAPI_CHECK_FSM_READY_BUSY == Fapi_Status_FsmReady) {
			done = true;
		}
		portRESET_PRIVILEGE( xRunningPrivileged );
		portENABLE_INTERRUPTS();
	} while (!done);

}


void flash_test(void) {
	portBASE_TYPE xRunningPrivileged;

	// uint16_t crc;
	Fapi_StatusType fapi_status;

	sprintf(respBuffer, "\r\nfapi init\r\n");
	serial_console_print(respBuffer);

	portDISABLE_INTERRUPTS();
	xRunningPrivileged = prvRaisePrivilege();
	fapi_status = Fapi_initializeFlashBanks((uint32)TI_FEE_OPERATING_FREQUENCY);
	portRESET_PRIVILEGE( xRunningPrivileged );
	portENABLE_INTERRUPTS();

	if(fapi_status != Fapi_Status_Success) {
		sprintf(respBuffer, "\r\nfapi init error: %d\r\n", fapi_status);
		serial_console_print(respBuffer);
		goto cleanup_exit;
	}



	sprintf(respBuffer, "fapi set active\r\n");
	serial_console_print(respBuffer);

	portDISABLE_INTERRUPTS();
	xRunningPrivileged = prvRaisePrivilege();
	fapi_status = Fapi_setActiveFlashBank(Fapi_FlashBank7);
	portRESET_PRIVILEGE( xRunningPrivileged );
	portENABLE_INTERRUPTS();

	if(fapi_status != Fapi_Status_Success) {
		sprintf(respBuffer, "\r\nfapi set active error: %d\r\n", fapi_status);
		serial_console_print(respBuffer);
		goto cleanup_exit;
	}




	sprintf(respBuffer, "fapi enable eeprom\r\n");
	serial_console_print(respBuffer);

	portDISABLE_INTERRUPTS();
	xRunningPrivileged = prvRaisePrivilege();
	Fapi_enableEepromBankSectors(0xFFFFFFFFU, 0xFFFFFFFFU);
	portRESET_PRIVILEGE( xRunningPrivileged );
	portENABLE_INTERRUPTS();



cleanup_exit:
#if 0
	portDISABLE_INTERRUPTS();
	xRunningPrivileged = prvRaisePrivilege();
	fapi_status = Fapi_doMarginReadByByte((uint8_t *)0xF0200000, (uint8_t *)&crc, 2, Fapi_NormalRead);
	portRESET_PRIVILEGE( xRunningPrivileged );
	portENABLE_INTERRUPTS();

	if(fapi_status != Fapi_Status_Success) {
		sprintf(respBuffer, "\r\nfapi read error: %d\r\n", fapi_status);
		serial_console_print(respBuffer);
		goto cleanup_exit;
	}
	sprintf(respBuffer, "new crc value: %d\r\n", crc);
	serial_console_print(respBuffer);
#endif

	sprintf(respBuffer, "\r\nftst finsihed.\r\n");
}



void wait_for_FEE(void) {
	portBASE_TYPE xRunningPrivileged;

	TI_FeeModuleStatusType fee_status = UNINIT;
	do {
		portDISABLE_INTERRUPTS();
		xRunningPrivileged = prvRaisePrivilege();
		TI_Fee_MainFunction();
		portRESET_PRIVILEGE( xRunningPrivileged );
		portENABLE_INTERRUPTS();

		fee_status = TI_Fee_GetStatus(0);
		DelayMs(500);
		serial_console_print(".");
	} while (fee_status != IDLE);
}

/* This is a basic test command to try out storing
 * and loading data written to flash memory.
 * It will corrupt valid configuration data.
 * This function demonstrates the basic API for
 * reading and writing pages to flash memory.
 */
void FEE_test(void)
{
	TI_FeeModuleStatusType fee_status = IDLE;
	Std_ReturnType err = E_OK;
	portBASE_TYPE xRunningPrivileged;

	TI_Fee_ErrorCodeType err_code;
	uint16_t crc;

	sprintf(respBuffer, "\r\nftst command start\r\n");
	serial_console_print(respBuffer);

	
	portDISABLE_INTERRUPTS();
	xRunningPrivileged = prvRaisePrivilege();
	TI_Fee_Init();
	portRESET_PRIVILEGE( xRunningPrivileged );
	portENABLE_INTERRUPTS();

	err_code = TI_FeeErrorCode(0);
	sprintf(respBuffer, "FEE error code after init: %d\r\n", err_code);
	serial_console_print(respBuffer);
	sprintf(respBuffer, "after init\r\n");
	serial_console_print(respBuffer);
	wait_for_FEE();
#if 0
	if (err_code != Error_Nil) {
	portDISABLE_INTERRUPTS();
	xRunningPrivileged = prvRaisePrivilege();
	err = TI_Fee_Format(0xA5A5A5A5U);
	portRESET_PRIVILEGE( xRunningPrivileged );
	portENABLE_INTERRUPTS();

	if (err != E_OK) {
		err_code = TI_FeeErrorCode(0);
		sprintf(respBuffer, "format failed, FEE error code: %d\r\n", err_code);
		serial_console_print(respBuffer);
	}
	wait_for_FEE();
	}
	sprintf(respBuffer, "after format\r\n");
	serial_console_print(respBuffer);
#endif
	fee_status = TI_Fee_GetStatus(0);
	if (fee_status != IDLE) {
		sprintf(respBuffer, "\r\nFEE is busy after init.\r\n");
		serial_console_print(respBuffer);
		goto cleanup_exit;
	}

	portDISABLE_INTERRUPTS();
	xRunningPrivileged = prvRaisePrivilege();
	err = TI_Fee_Read(1, 0, (uint8_t *)&crc, 2);
	portRESET_PRIVILEGE( xRunningPrivileged );
	portENABLE_INTERRUPTS();

	if (err != E_OK) {

		sprintf(respBuffer, "\r\nFEE read error: %d\r\n", err);
		serial_console_print(respBuffer);
		fee_status = TI_Fee_GetStatus(0);
		sprintf(respBuffer, "FEE after read error status: %d\r\n", fee_status);
		serial_console_print(respBuffer);

		err_code = TI_FeeErrorCode(0);
		sprintf(respBuffer, "FEE error code: %d\r\n", err_code);
		serial_console_print(respBuffer);


#if 0
		err = TI_Fee_EraseImmediateBlock(1);
		if (err) {
			sprintf(respBuffer, "\r\nFEE erase error: %d\r\n", err);
			serial_console_print(respBuffer);
		}
		wait_for_FEE();

#endif
		crc = 0;
	//	sprintf(respBuffer, "exit due to error.\r\n", err);
		// goto cleanup_exit;
	}
	sprintf(respBuffer, "\r\nvalue: %d\r\n", crc);
	serial_console_print(respBuffer);

	portDISABLE_INTERRUPTS();
	xRunningPrivileged = prvRaisePrivilege();
	err = TI_Fee_EraseImmediateBlock(1);
	portRESET_PRIVILEGE( xRunningPrivileged );
	portENABLE_INTERRUPTS();

	sprintf(respBuffer, "after erase\r\n");
	serial_console_print(respBuffer);

	if (err != E_OK) {
		sprintf(respBuffer, "\r\nFEE erase error: %d\r\n", err);
		serial_console_print(respBuffer);
		goto cleanup_exit;
	}
	wait_for_FEE();
	
	crc++;
	sprintf(respBuffer, "before write\r\n");
	serial_console_print(respBuffer);

	portDISABLE_INTERRUPTS();
	xRunningPrivileged = prvRaisePrivilege();
	err = TI_Fee_WriteSync(1, (uint8_t *)&crc);
	portRESET_PRIVILEGE( xRunningPrivileged );
	portENABLE_INTERRUPTS();

	sprintf(respBuffer, "after write\r\n");
	serial_console_print(respBuffer);

	if (err != E_OK) {
		sprintf(respBuffer, "\r\nFEE write error: %d\r\n", err);
		serial_console_print(respBuffer);
		goto cleanup_exit;
	}



cleanup_exit:
	wait_for_FEE();

	portDISABLE_INTERRUPTS();
	xRunningPrivileged = prvRaisePrivilege();
	err = TI_Fee_Shutdown();
	portRESET_PRIVILEGE( xRunningPrivileged );
	portENABLE_INTERRUPTS();

	if (fee_status != E_OK) {
		sprintf(respBuffer, "\r\nFEE shutdown error: %d\r\n", err);
		serial_console_print(respBuffer);
	}
	sprintf(respBuffer, "\r\nftst finsihed.\r\n");
}








/* Added user code to source/sci.c */
void sciEnableLineTransferMode(void);
void sciDisableLineTransferMode(void);

void console_sciNotification(sciBASE_t *sci, uint32 flags)
{
	unsigned char dummy;
    if (sci == UART) {
        if (flags == SCI_RX_INT) {
            xQueueSendFromISR( xSCIReceiveStatusQueue, &dummy, NULL );
        } else if (flags == SCI_TX_INT) {
            xQueueSendFromISR( xSCITransmitStatusQueue, &dummy, NULL );
        }
        /* TODO: handle other errors */
    }
}


void serial_console_receive_task( void *pvParameters )
{
    uint8 dummy;
    char input_line[INPUT_LINE_MAX_LENGTH + 1];
    uint8 curr_line_len = 0;
    sciEnableLineTransferMode();
	for( ;; )
    {
		memset(input_line, 0, INPUT_LINE_MAX_LENGTH + 1);
        sciReceive(UART, 1, (uint8_t *)&(input_line[curr_line_len]));
        xQueueReceive( xSCIReceiveStatusQueue, &dummy, portMAX_DELAY );
		if (input_line[0] == 'f') {
			serial_console_print("F021 Flash test\r\n");
			flash_test();
		} else if (input_line[0] == 'e') {
			serial_console_print("FEE test\r\n");
			FEE_test();
		} else {
			serial_console_print("G: ");
			serial_console_print(input_line);
		}
        serial_console_print("\r\n");
    }
}


/* Assumes a NULL-terminated string.
 * Note that this can block for quite a while
 * if other tasks are also printing.
 */
void serial_console_print( char *text_line )
{
    uint8 dummy;
    xSemaphoreTake (xConsolePrintMutex, portMAX_DELAY);
    sciSend (UART, strlen((const char *)text_line), (uint8_t *)text_line);
    xQueueReceive (xSCITransmitStatusQueue, &dummy, portMAX_DELAY);
    xSemaphoreGive (xConsolePrintMutex);
}
